function gamma = omp(varargin)
%OMP Sparsity-constrained Orthogonal Matching Pursuit.
%  GAMMA = OMP(D,X,G,L) solves the optimization problem
%
%       min  |X - D*GAMMA|_2     s.t.  |GAMMA|_0 <= L
%      gamma
%
%  for each of the signals in X, using the OMP method. Here, D is the
%  dictionary, X is a matrix containing the column signals, L is the # of
%  non-zero coefficients in each signal representation, and GAMMA is a
%  matrix containing the sparse representations as its columns. G is the
%  Gramm matrix D'*D. The dictionary is assumed to be normalized.
%
%  GAMMA = OMP(D,X,[],L) performs the same computation, but does not
%  require the matrix G. In this case OMP computes the relevant columns of
%  G on the fly, thus reducing memory requirements substantially when the
%  dictionary is highly overcomplete. This form is significantly slower
%  (typically 2-4 times slower), and should only be used when G is too
%  large to be stored in memory.
%
%  GAMMA = OMP(DtX,G,L) is the fastest implementation of OMP, but it
%  requires the most memory. Here, DtX contains the application of D' on X,
%  i.e. DtX = D'*X. Compared to the other two versions, this one does not
%  require OMP to compute D'*X itself, and is thus significantly quicker.
%  Note that in general, the call sequence
%
%    DtX = D'*X;
%    GAMMA = OMP(DtX,G,L);
%
%  will be faster than the single call
%
%    GAMMA = OMP(D,X,G,L);
%
%  due to optimized matrix multiplications in Matlab. Of course, computing
%  the matrix D'*X explicitly may not be possible for large data sets or
%  highly overcomplete dictionaries. Therefore, one of the two 4-parameter
%  versions should be used when memory resources are limited; both of those
%  versions compute D'*X for just one signal at a time, and thus require
%  much less memory.
%
%  GAMMA = OMP(...,PARAM1,VAL1,PARAM2,VAL2,...) specifies additional
%  parameters for OMP. Available parameters are:
%
%    'gammamode' - Used to specify the type of the output parameter GAMMA.
%                  Can be either 'full' or 'sparse', corresponding to a
%                  full or sparse matrix, respectively. By default, GAMMA
%                  is returned as a sparse matrix.
%    'messages'  - Specifies whether messages should be displayed during
%                  the coding. When positive, specifies the number of
%                  seconds between status prints. When negative, no
%                  messages are displayed (default behavior).
%    'profile'   - Can either be 'on' or 'off'. When 'on', profiling
%                  information is collected during the function operation,
%                  and printed when its done.
%
%
%  Summary of OMP versions:
%
%    version      |   speed     |   memory
%  --------------------------------------------------
%   OMP(DtX,G,L)  |  very fast  |  very large
%   OMP(D,X,G,L)  |  fast       |  moderate to large
%   OMP(D,X,[],L) |  very slow  |  small
%  --------------------------------------------------
%
%
%  References:
%  [1] M. Elad, R. Rubinstein, and M. Zibulevsky, "Efficient Implementation
%      of the K-SVD Algorithm using Batch Orthogonal Matching Pursuit",
%      Technical Report - CS, Technion, April 2008.
%
%  See also OMP2, OMPDENOISE.


%  Ron Rubinstein
%  Computer Science Department
%  Technion, Haifa 32000 Israel
%  ronrubin@cs
%
%  April 2008


% default options

sparse_gamma = 1;
msgdelta = -1;
nonneg = 0;
profile = 0;


% determine number of parameters

paramnum = 1;
while (paramnum<=nargin && ~ischar(varargin{paramnum}))
  paramnum = paramnum+1;
end
paramnum = paramnum-1;


% parse options

for i = paramnum+1:2:length(varargin)
  paramname = varargin{i};
  paramval = varargin{i+1};

  switch lower(paramname)

    case 'gammamode'
      if (strcmpi(paramval,'sparse'))
        sparse_gamma = 1;
      elseif (strcmpi(paramval,'full'))
        sparse_gamma = 0;
      else
        error('Invalid GAMMA mode');
      end

    case 'messages'
      msgdelta = paramval;
      
    case 'profile'
      if (strcmpi(paramval,'on'))
        profile = 1;
      elseif (strcmpi(paramval,'off'))
        profile = 0;
      else
        error('Invalid profile mode');
      end

    otherwise
      error(['Unknown option: ' paramname]);
  end
  
end


% determine call type

if (paramnum==3)
  DtX = varargin{1};
  G = varargin{2};
  L = varargin{3};
  D = [];
  X = [];
elseif (paramnum==4)
  D = varargin{1};
  X = varargin{2};
  G = varargin{3};
  L = varargin{4};
  DtX = [];
else
  error('Invalid number of parameters');
end


% omp

gamma = ompmex(D,X,DtX,G,L,sparse_gamma,msgdelta,nonneg,profile);
