function gamma = omp2(varargin)
%OMP2 Error-constrained Orthogonal Matching Pursuit.
%  GAMMA = OMP2(D,X,G,EPSILON) solves the optimization problem
%
%       min  |GAMMA|_0     s.t.  |X - D*GAMMA|_2 <= EPSILON
%      gamma
%
%  for each of the signals in X, using the OMP method. Here, D is the
%  dictionary, X is a matrix containing the columns signal, EPSILON is the
%  error target for each signal, and GAMMA is a matrix containing the
%  sparse representations as its columns. G is the Gramm matrix D'*D. The
%  dictionary is assumed to be normalized.
%
%  GAMMA = OMP2(D,X,[],EPSILON) performs the same computation, but does not
%  require the matrix G. In this case OMP2 computes the relevant columns of
%  G on the fly, thus reducing memory requirements substantially when the
%  dictionary is highly overcomplete. Note that this form computes the
%  relevant columns of G for each signal individually, and thus the same
%  columns may be recomputed for many signals. This call to OMP2 is
%  therefore significantly slower than all other calls (at least 2-4 times
%  slower, and commonly even more), and should only be used when G is too
%  large to be stored in memory.
%
%  GAMMA = OMP2(DtX,XtX,G,EPSILON) is the fastest implementation of OMP2,
%  but it requires the most memory. Here, DtX is the application of D' on
%  X, i.e. DtX = D'*X, and XtX is a row vector containing the squared norms
%  of the signals in X, sum(X.*X). Compared to the other two versions,
%  this one does not require OMP2 to compute DtX and XtX itself, and is
%  thus significantly quicker. Note that in general, the call
%
%    GAMMA = OMP2(D'*X, sum(X.*X), G, EPSILON);
%
%  will be faster than the call
%
%    GAMMA = OMP2(D,X,G,EPSILON);
%
%  due to optimized matrix multiplications in Matlab. Of course, computing
%  the matrix D'*X explicitly may not be possible for large data sets or
%  highly overcomplete dictionaries. Therefore, one of the two 4-parameter
%  versions should be used when memory resources are limited; both of those
%  versions compute D'*X for just one signal at a time, and thus require
%  much less memory.
%
%  GAMMA = OMP2(...,PARAM1,VAL1,PARAM2,VAL2,...) specifies additional
%  parameters for OMP2. Available parameters are:
%
%    'gammamode' - Used to specify the type of the output parameter GAMMA.
%                  Can be either 'full' or 'sparse', corresponding to a
%                  full or sparse matrix, respectively. By default, GAMMA
%                  is returned as a sparse matrix.
%    'maxatoms' -  Specifies the maximum number of atoms allowed for
%                  representing each signal. If specified, OMP2 stops
%                  selecting atoms once this bound is met, even if the
%                  error target has not been reached.
%    'messages'  - Specifies whether messages should be displayed during
%                  the coding. When positive, specifies the number of
%                  seconds between status prints. When negative, no
%                  messages are displayed (default behavior).
%    'profile'   - Can either be 'on' or 'off'. When 'on', profiling
%                  information is collected during the function operation,
%                  and printed when its done.
%
%
%  Summary of OMP2 versions:
%
%    version                 |   speed     |   memory
%  -------------------------------------------------------------
%   OMP2(DtX,XtX,G,EPSILON)* |  very fast  |  very large
%   OMP2(D,X,G,EPSILON)      |  fast       |  moderate to large
%   OMP2(D,X,[],EPSILON)     |  very slow  |  small
%  -------------------------------------------------------------
%
%
%  References:
%  [1] M. Elad, R. Rubinstein, and M. Zibulevsky, "Efficient Implementation
%      of the K-SVD Algorithm using Batch Orthogonal Matching Pursuit",
%      Technical Report - CS, Technion, April 2008.
%
%  See also OMP, OMPDENOISE.


%  Ron Rubinstein
%  Computer Science Department
%  Technion, Haifa 32000 Israel
%  ronrubin@cs
%
%  April 2008


% default options

sparse_gamma = 1;
msgdelta = -1;
maxatoms = -1;
nonneg = 0;
profile = 0;


% determine number of parameters

paramnum = 1;
while (paramnum<=nargin && ~ischar(varargin{paramnum}))
  paramnum = paramnum+1;
end
paramnum = paramnum-1;


% parse options

for i = paramnum+1:2:length(varargin)
  paramname = varargin{i};
  paramval = varargin{i+1};

  switch lower(paramname)

    case 'gammamode'
      if (strcmpi(paramval,'sparse'))
        sparse_gamma = 1;
      elseif (strcmpi(paramval,'full'))
        sparse_gamma = 0;
      else
        error('Invalid GAMMA mode');
      end
      
    case 'maxatoms'
      maxatoms = paramval;

    case 'messages'
      msgdelta = paramval;

    case 'profile'
      if (strcmpi(paramval,'on'))
        profile = 1;
      elseif (strcmpi(paramval,'off'))
        profile = 0;
      else
        error('Invalid profile mode');
      end

    otherwise
      error(['Unknown option: ' paramname]);
  end
  
end


% determine call type

if (paramnum==4)
  
  n1 = size(varargin{1},1);
  n2 = size(varargin{2},1);
  n3 = size(varargin{3},1);
  
  if ( (n1>1 && n2==1) || (n1==1 && n2==1 && n3==1) )  %  DtX,XtX,G,EPSILON
    
    DtX = varargin{1};
    XtX = varargin{2};
    G = varargin{3};
    epsilon = varargin{4};
    D = [];
    X = [];
    
  else  % D,X,G,EPSILON
    
    D = varargin{1};
    X = varargin{2};
    G = varargin{3};
    epsilon = varargin{4};
    DtX = [];
    XtX = [];
    
  end
  
elseif (nonneg && paramnum==5)
  
  D = varargin{1};
  X = varargin{2};
  DtX = varargin{3};
  G = varargin{4};
  epsilon = varargin{5};
  
else
  error('Invalid number of parameters');
end


% omp

gamma = omp2mex(D,X,DtX,XtX,G,epsilon,sparse_gamma,msgdelta,maxatoms,nonneg,profile);

